/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <netinet/in.h>
#include <string.h>

#include <gtk/gtk.h>

#include "main.h"
#include "dc210.h"
#include "dchandle.h"
#include "dialog.h"
#include "lib.h"
#include "progress.h"
#include "xpm.h"

struct camera_stat status;
int timeoutflag;

unsigned char *batteryStatus[] = {"full", "low", "empty"};
unsigned char *flashModes[] = {"auto", "fill", "off", "auto-read-eye", "fill read-eye"};
unsigned char *cameraTypes[] = {"", "Kodak DC50", "Kodak DC120", "", "Kodak DC210 Zoom"};
unsigned char *compressionModes[] = {"none", "low", "medium", "high"};
unsigned char *resolution[] = {"640x480", "1152x864"};
unsigned char *acStatus[] = {"not connected", "connected", "N/A"};

#ifdef USE_BMP
unsigned char header[] = {
   0x42, 0x4d, 0x36, 0x51,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28,
   0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x48,
   0x00, 0x00, 0x00, 0x01, 0x00, 24, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x51, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
#else
unsigned char header[] = {
   0x50, 0x36, 0x0A, 0x39, 0x36, 0x20, 0x37, 0x32,
   0x0A, 0x32, 0x35, 0x35, 0x0A
};
#endif
   
int init_dc210()
{
   int ret_val;

   init_dchandle();

   ret_val = send_command(DC_INITIALIZE, 0, 0, 0, 0);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   ret_val = set_port_speed();
   if (ret_val == -1)
     return -1;

   ret_val = send_command(DC_INITIALIZE, 0, 0, 0, 0);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   ret_val = load_status();
   if (ret_val == -1)
     return -1;

   return 0;
}

void destroy_dc210()
{
   int ret_val;

   ratio_no = 9600;
   ret_val = set_port_speed();
   if (ret_val == -1)
     return;

   close_dchandle();
}

int load_status()
{
   unsigned char *data = NULL;
   int ret_val;

   ret_val = send_command(DC_STATUS, 0, 0, 0, 0);
   if (ret_val == -1)
     return -1;

   data = safe_malloc(256);
   ret_val = read_packet(data, 256);
   if (ret_val == -1)
     {
	safe_free((void **)&data);
	return -1;
     }

   ret_val = command_complete();
   if (ret_val == -1)
     {
	safe_free((void **)&data);
	return -1;
     }

   memcpy (&status, data, sizeof(status));

   safe_free((void **)&data);
   return 0;
}

unsigned char *get_resolution(int id)
{
   return resolution[id];
}

unsigned char *get_flashmode(int id)
{
   return flashModes[id];
}

unsigned char *get_comprmode(int id)
{
   return compressionModes[id];
}

unsigned char *get_cameratype(int id)
{
   return cameraTypes[id];
}

unsigned char *get_batterystat(int id)
{
   return batteryStatus[id];
}

unsigned char *get_acstat(int id)
{
   if (id != 0 && id != 1)
     return acStatus[2];
   else
     return acStatus[id];
}

unsigned char *get_cameratime(unsigned char *s)
{
   time_t t;
   long ht;

   ht = ntohl(status.cameraTime);

   t = (time_t)(CAMERAEPOC + (long)(ht / 2));
   strftime(s, 80, "%a, %d %b %Y %T", localtime(&t));
   return s;
}

int get_picture_info(int no, struct pinfo_lst *plst)
{
   int num = (int)status.numPictures;
   unsigned char msb, lsb;
   unsigned char *data = NULL;
   struct picture_info pinfo, *p;
   long hl;
   int ret_val;

   if ((no < 1) || (no > num))
     {
	return -1;
     }

   no = no - 1;
   msb = (unsigned char)((no & 0xff00)>>8);
   lsb = (unsigned char)(no & 0xff);
   ret_val = send_command(DC_PICTURE_INFO, msb, lsb, 0x00, 0x00);
   if (ret_val == -1)
     {
	return -1;
     }

   data = safe_malloc(256);
   ret_val = read_packet(data, 256);
   if (ret_val == -1)
     {
	safe_free((void **)&data);
	return -1;
     }

   ret_val = command_complete();
   if (ret_val == -1)
     {
	safe_free((void **)&data);
	return -1;
     }

   p = &pinfo;
   memcpy (p, data, sizeof(pinfo));

   sprintf(plst->number, "%d", p->pictureNumber + 1);
   hl = ntohl(p->fileSize);
   sprintf(plst->filesize, "%ld", hl);
   strcpy(plst->filename, p->fileName);
   strcpy(plst->compression, get_comprmode(p->compression));
   strcpy(plst->resolution, get_resolution(p->resolution));

   safe_free((void **)&data);
   return 0;
}

static void
get_pic_cancel (void)
{
  progress_canceled = 1;
}

int get_picture(int no, struct pinfo_lst *plst, unsigned char *fname)
{
   int num = (int)status.numPictures;
   unsigned char msb, lsb, *pict;
   unsigned char label[80];
   int ret_val, rdcnt, filesize;
   int fd;

   if ((no < 1) || (no > num))
     {
	return -1;
     }

   if ( (fd = open(fname, O_WRONLY | O_CREAT,
		   S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1 )
     {
	return -1;
     }

   no = no - 1;
   msb = (unsigned char)((no & 0xff00)>>8);
   lsb = (unsigned char)(no & 0xff);
   ret_val = send_command(DC_PICTURE_DOWNLOAD, msb, lsb, 0x00, 0x00);
   if (ret_val == -1)
     {
	return -1;
     }

   pict = safe_malloc(1024);
   filesize = atoi(plst->filesize);
   rdcnt = 0;
   progress_canceled = 0;

   if (progress_box)
     progress_free (progress_box);
   sprintf(label, "Saving Image No %d\nFilename: %s", no + 1, fname);
   progress_box = progress_new ("Downloading Image", label,
				(GtkSignalFunc) get_pic_cancel, 0);

   while (rdcnt < filesize)
     {
	ret_val = read_packet(pict, 1024);
	if (ret_val == -1)
	  {
	     progress_free (progress_box);
	     progress_box = NULL;
	     safe_free((void **)&pict);
	     close(fd);
	     return -1;
	  }

	progress_update (progress_box, rdcnt / (gfloat) filesize);
	while (gtk_events_pending ())
	  gtk_main_iteration ();

	if (progress_canceled == 1)
	  break;

	rdcnt += 1024;
	if ( rdcnt > filesize )
	  {
	     write(fd, pict, filesize % 1024);
	  }
	else
	  {
	     write(fd, pict, 1024);
	  }
     }
   close(fd);

   while (gtk_events_pending ())
     gtk_main_iteration ();

   progress_free (progress_box);
   progress_box = NULL;
   safe_free((void **)&pict);

   if (progress_canceled == 1)
     {
	return -1;
     }

   ret_val = command_complete();
   if (ret_val == -1)
     {
	return -1;
     }

   return 0;
}

int set_camera_id(unsigned char *cameraID)
{
   int ret_val;

   ret_val = send_command(DC_SET_CAMERA_ID, 0, 0, 0, 0);
   if (ret_val == -1)
     return -1;

   ret_val = send_packet(PKT_CTRL_EOF, cameraID, 58);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   return 0;
}

int take_picture()
{
   int ret_val;

   ret_val = send_command(DC_TAKE_PICTURE, 0, 0, 0, 0);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   ret_val = load_status();
   if (ret_val == -1)
     return -1;

   return 0;
}

int del_picture(int no)
{
   int num = (int)status.numPictures;
   unsigned char msb, lsb;
   int ret_val;

   if ((no < 1) || (no > num))
     {
	return -1;
     }

   no = no - 1;
   msb = (unsigned char)((no & 0xff00)>>8);
   lsb = (unsigned char)(no & 0xff);
   ret_val = send_command(DC_ERASE_IMAGE_IN_CARD, msb, lsb, 0x00, 0x00);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   ret_val = load_status();
   if (ret_val == -1)
     return -1;

   return 0;
}

int del_all_picture()
{
   int ret_val;

   ret_val = send_command(DC_ERASE_IMAGE_IN_CARD, 0xFF, 0xFF, 0x00, 0x00);
   if (ret_val == -1)
     return -1;

   ret_val = command_complete();
   if (ret_val == -1)
     return -1;

   ret_val = load_status();
   if (ret_val == -1)
     return -1;

   return 0;
}

static void
get_thumb_cancel (void)
{
  progress_canceled = 1;
}

int get_thumbnail(int no, struct pinfo_lst *plst)
{
   int num = (int)status.numPictures;
   unsigned char msb, lsb, *pict, *picture;
   unsigned char *fname, *p;
   unsigned char label[80];
   int ret_val, rdcnt, filesize;

   if ((no < 1) || (no > num))
     {
	return -1;
     }

   no = no - 1;
   msb = (unsigned char)((no & 0xff00)>>8);
   lsb = (unsigned char)(no & 0xff);
   filesize = 20736;
   ret_val = send_command(DC_PICTURE_THUMBNAIL, msb, lsb, 1, 0x00);
   if (ret_val == -1)
     {
	return -1;
     }

   pict = picture = safe_malloc(filesize + (1024 - (filesize % 1024)));
   rdcnt = 0;
   progress_canceled = 0;

   if (progress_box)
     progress_free (progress_box);
   sprintf(label, "Getting thumbnail No %d", no + 1);
   progress_box = progress_new ("Downloading Image", label,
				(GtkSignalFunc) get_thumb_cancel, 0);

   while (rdcnt < filesize)
     {
	ret_val = read_packet(pict, 1024);
	if (ret_val == -1)
	  {
	     progress_free (progress_box);
	     progress_box = NULL;
	     safe_free((void **)&picture);
	     return -1;
	  }

	progress_update (progress_box, rdcnt / (gfloat) filesize);
	while (gtk_events_pending ())
	  gtk_main_iteration ();

	if (progress_canceled == 1)
	  break;

	rdcnt += 1024;
	pict += 1024;
     }

   while (gtk_events_pending ())
     gtk_main_iteration ();

   progress_free (progress_box);
   progress_box = NULL;
   safe_free((void **)&pict);

   if (progress_canceled == 1)
     {
	return -1;
     }

   ret_val = command_complete();
   if (ret_val == -1)
     {
	return -1;
     }

   fname = safe_malloc(SHORT_STRING);
   strcpy(fname, dir_str);
   strcat(fname, plst->filename);
   p = strrchr(fname, '.');
   *p = '\0';
   strcat(fname, ".xpm");
   
   save_xpm(fname, picture);
   thumbnail_dialog(fname);

   safe_free((void **)&fname);

   return 0;
}
