/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "main.h"
#include "dchandle.h"
#include "dialog.h"
#include "lib.h"

int dc_handle;
sigjmp_buf env;

void timed_out(int sig)
{ 
   siglongjmp(env, 1);
}

int init_dchandle()
{
   char *temp = NULL;

   if ( (dc_handle = open(device_str, O_RDWR)) == -1 )
     {
	safe_free((void **)&device_str);
	return -1;
     }

   temp = safe_malloc(SHORT_STRING);
   sprintf(temp, "stty raw -onlcr -iexten -echonl -noflsh -tostop -echoprt -ixoff -olcuc -ocrnl -onocr -onlret -ofill -ofdel -echo -echoe -echok -echoctl -echoke min 255 time 5 cs8 9600 < %s", device_str);
   system(temp);

   safe_free((void **)&temp);
   return 0;
}

void close_dchandle()
{
   close(dc_handle);
}

int set_port_speed()
{
   char *temp = NULL;
   unsigned char cmd1, cmd2;

   switch (ratio_no)
     {
      case 9600 :
	cmd1 = 0x96;
	cmd2 = 0x00;
	break ;
      case 19200 :
	cmd1 = 0x19;
	cmd2 = 0x20;
	break ;
      case 38400 :
	cmd1 = 0x38;
	cmd2 = 0x40;
	break ;
      case 57600 :
	cmd1 = 0x57;
	cmd2 = 0x60;
	break ;
      case 115200 :
	cmd1 = 0x11;
	cmd2 = 0x52;
	break ;
      default:
	cmd1 = 0x96;
	cmd2 = 0x00;
	break ;
     }
   send_command(DC_SET_SPEED, cmd1, cmd2, 0, 0);

   temp = safe_malloc(SHORT_STRING);
   sprintf(temp, "stty raw min 255 time 5 cs8 %s < %s", ratio_str, device_str);
   system(temp);

   safe_free((void **)&temp);

   return 0;
}


int read_byte(unsigned char *data)
{
   ssize_t rdcnt;
   char *temp = NULL;

   (void)signal(SIGALRM, timed_out);

   if (sigsetjmp(env, SIGALRM) == 0)
     {
	alarm(5);
	do
	  {
	     rdcnt = read(dc_handle, data, 1);
	  }
	while (rdcnt == 0);
	alarm(0);
     }
   else
     {
	alarm(0);
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "camera initialization is timed out\nould not initialize camera %s / %d", device_str, ratio_no);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }

   if (rdcnt != 1)
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "wanted to read one byte, read (%d)", rdcnt);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }
   return 0;
}

int write_byte(unsigned char data)
{
   ssize_t wrtcnt;
   char *temp = NULL;

   do
     {
	wrtcnt = write(dc_handle, &data, 1);
     }
   while (wrtcnt == 0);

   if (wrtcnt != 1)
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "wanted to write one byte, wrote (%d)", wrtcnt);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }
   return 0;
}

int send_command(unsigned char command, 
		 unsigned char arg1,
		 unsigned char arg2,
		 unsigned char arg3,
		 unsigned char arg4)
{
   unsigned char command_str[8];
   ssize_t wrtcnt;
   int ret_val;
   char *temp = NULL;
   unsigned char ack;

   command_str[0] = command;
   command_str[1] = 0x00;
   command_str[2] = arg1;
   command_str[3] = arg2;
   command_str[4] = arg3;
   command_str[5] = arg4;
   command_str[6] = 0x00;
   command_str[7] = 0x1A;

   wrtcnt = write(dc_handle, command_str, 8);
   if (wrtcnt != 8)
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "only wrote partial command (%d)", wrtcnt);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }

   ret_val = read_byte(&ack);
   if (ret_val == -1)
     return -1;

   if (ack != DC_COMMAND_ACK)
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "bad ack from camera (%0X)", ack);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }
   return 0;
}

int send_correct_packet()
{
   int ret_val;
   unsigned char data;
   char *temp = NULL;

   data = DC_CORRECT_PACKET;
   ret_val = write_byte(data);

   ret_val = read_byte(&data);
   if (ret_val == -1)
     return -1;

   if ((data != 0) && (data != 1))
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "bad response (%0X)", data);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     } 

   return 0;
}
   
unsigned char
checksum(unsigned char *data, int length)
{
   unsigned char sum = 0;

   while( length-- > 0 )
     {
	sum ^= *data++;
     }

   return sum;
}

int send_packet( unsigned char cb,
		 unsigned char *data,
		 int length)
{
   int ret_val;
   int write_cnt;
   unsigned char ack;
   unsigned char sum;
   ssize_t wrtcnt = 0;
   unsigned char *data_ptr;
   char *temp = NULL;

   ret_val = write_byte(cb);
   if (ret_val == -1)
     return -1;

   data_ptr = data;
   write_cnt = 0;
   while (write_cnt < length)
     {
	wrtcnt = write(dc_handle, data_ptr, length - write_cnt);
	data_ptr += wrtcnt;
	write_cnt += wrtcnt;
     }

   sum = checksum(data, length);
   ret_val = write_byte(sum);
   if (ret_val == -1)
     return -1;

   ret_val = read_byte(&ack);
   if (ret_val == -1)
     return -1;

   if ( ack != DC_CORRECT_PACKET )
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "camera didn't like the packet (%0X)", ret_val);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }
   return 0;
}

int command_complete()
{
   int ret_val;
   unsigned char data;
   char *temp = NULL;

   do
     {
	ret_val = read_byte(&data);
	if (ret_val == -1)
	  return -1;
     }
   while (data == DC_BUSY);

   if (data != DC_COMMAND_COMPLETE)
     {
	temp = safe_malloc(SHORT_STRING);
	if (data == DC_ILLEGAL_PACKET)
	  {
	     sprintf(temp, "illegal packet received from host");
	  }
	else
	  {
	     sprintf(temp, "command not completed (%0X)", data);
	  }
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }

   return 0;
}

int read_packet(unsigned char *data_str, int length)
{
   int ret_val;
   int read_cnt;
   unsigned char sum;
   unsigned char data;
   unsigned char *data_ptr;
   ssize_t rdcnt = 0;
   char *temp = NULL;

   ret_val = read_byte(&data);
   if (ret_val == -1)
     return -1;

   if ( data != 0x01 )
     {
	temp = safe_malloc(SHORT_STRING);
	sprintf(temp, "packet control byte invalid (%0X)", data);
	alert_dialog(temp, "OK", NULL, NULL);
	safe_free((void **)&temp);
	return -1;
     }

   data_ptr = data_str;
   read_cnt = 0;
   while (read_cnt < length)
     {
	rdcnt = read(dc_handle, data_ptr, length - read_cnt);
	data_ptr += rdcnt;
	read_cnt += rdcnt;
     }

   ret_val = read_byte(&data);
   if (ret_val == -1)
     return -1;

   sum = checksum(data_str, length);
   if ( sum != data )
     {
	ret_val = write_byte(DC_ILLEGAL_PACKET);
	return -1;
     }
   else
     {
	ret_val = write_byte(DC_CORRECT_PACKET);
	return 0;
     }
}
