/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "main.h"
#include "menu.h"
#include "list.h"
#include "dialog.h"
#include "lib.h"
#include "filesel.h"
#include "dc210.h"
#include "progress.h"

#ifndef GTK_HAVE_FEATURES_1_1_0
static void menus_remove_accel(GtkWidget * widget, gchar * signal_name, gchar * path);
static gint menus_install_accel(GtkWidget * widget, gchar * signal_name, gchar key, gchar modifiers, gchar * path);
void menus_init(void);
void menus_create(GtkMenuEntry * entries, int nmenu_entries);

static GtkMenuEntry menu_items[] =
{
     {"<Main>/File/Select Device...", "<control>D", menu_sel_device, NULL},
     {"<Main>/File/Select Ratio...", "<control>R", menu_sel_ratio, NULL},
     {"<Main>/File/Get Status...", "<control>G", menu_get_status, NULL},
     {"<Main>/File/Set Camera ID...", "<control>C", menu_set_cameraID, NULL},
     {"<Main>/File/Take A Picture", "<control>T", menu_take_picture, NULL},
     {"<Main>/File/Save Preference", "<control>S", menu_save_rc, NULL},
     {"<Main>/File/<separator>", NULL, NULL, NULL},
     {"<Main>/File/Delete All", "<control>S", menu_del_all, NULL},
     {"<Main>/File/<separator>", NULL, NULL, NULL},
     {"<Main>/File/Exit", "<control>X", menu_quit, NULL},
     {"<Main>/Help/About GKDCPI...", "<control>H", menu_about, NULL},
     {"<PopUp>/View Thumbnail", NULL, menu_view_thumb, NULL},
     {"<PopUp>/Delete Image", NULL, menu_del_image, NULL},
     {"<PopUp>/Save", NULL, menu_save_image, NULL},
     {"<PopUp>/Save As...", NULL, menu_save_as, NULL}
};

static int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
static int initialize = TRUE;
static GtkMenuFactory *factory;
static GtkMenuFactory *subfactory[1];
static GHashTable *entry_ht = NULL;

void init_menu()
{
   GtkWidget *menubar;
   GtkAcceleratorTable *accel;

   if (initialize)
     menus_init();

   menubar = subfactory[0]->widget;
   accel = subfactory[0]->table;
   context_menu = subfactory[1]->widget;

   gtk_window_add_accelerator_table(GTK_WINDOW(window), accel);
   gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
   menus_set_sensitive("<Main>/File/Take A Picture", FALSE);
   gtk_widget_show(menubar);
}

void menus_init(void)
{
   if (initialize)
     {
	initialize = FALSE;

	factory = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);
	subfactory[0] = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);
	subfactory[1] = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU);

	gtk_menu_factory_add_subfactory(factory, subfactory[0], "<Main>");
	gtk_menu_factory_add_subfactory(factory, subfactory[1], "<PopUp>");
	menus_create(menu_items, nmenu_items);
     }
}

void menus_create(GtkMenuEntry * entries, int nmenu_entries)
{
   char *accelerator;
   int i;

   if (initialize)
     menus_init();

   if (entry_ht)
     for (i = 0; i < nmenu_entries; i++)
     {
	accelerator = g_hash_table_lookup(entry_ht, entries[i].path);
	if (accelerator)
	  {
	     if (accelerator[0] == '\0')
	       entries[i].accelerator = NULL;
	     else
	       entries[i].accelerator = accelerator;
	  }
     }
   gtk_menu_factory_add_entries(factory, entries, nmenu_entries);

   for (i = 0; i < nmenu_entries; i++)
     if (entries[i].widget)
     {
	gtk_signal_connect(GTK_OBJECT(entries[i].widget),
			   "install_accelerator",
			   (GtkSignalFunc) menus_install_accel,
			   entries[i].path);
	gtk_signal_connect(GTK_OBJECT(entries[i].widget),
			   "remove_accelerator",
			   (GtkSignalFunc) menus_remove_accel,
			   entries[i].path);
     }
}

static gint menus_install_accel(GtkWidget * widget, gchar * signal_name,
				gchar key, gchar modifiers, gchar * path)
{
  char accel[64];
  char *t1, t2[2];
    
  accel[0] = '\0';
  if (modifiers & GDK_CONTROL_MASK)
    strcat(accel, "<control>");
  if (modifiers & GDK_SHIFT_MASK)
    strcat(accel, "<shift>");
  if (modifiers & GDK_MOD1_MASK)
    strcat(accel, "<alt>");

  t2[0] = key;
  t2[1] = '\0';
  strcat(accel, t2);

  if (entry_ht)
    {
      t1 = g_hash_table_lookup(entry_ht, path);
      g_free(t1);
    }
  else
    entry_ht = g_hash_table_new(g_str_hash, g_str_equal);

  g_hash_table_insert(entry_ht, path, g_strdup(accel));
  
  return TRUE;
}

static void menus_remove_accel(GtkWidget * widget,
			       gchar * signal_name,
			       gchar * path)
{
  char *t;

  if (entry_ht)
    {
      t = g_hash_table_lookup(entry_ht, path);
      g_free(t);
      g_hash_table_insert(entry_ht, path, g_strdup(""));
    }
}
#else
static GtkWidget *menubar = NULL;

static GtkItemFactoryEntry menu_items[] =
{
     {"/File", NULL, NULL, 0, "<Branch>"},
     {"/File/Select Device...", "<control>D", menu_sel_device, 0, NULL},
     {"/File/Select Ratio...", "<control>R", menu_sel_ratio, 0, NULL},
     {"/File/Get Status...", "<control>G", menu_get_status, 0, NULL},
     {"/File/Set Camera ID...", "<control>C", menu_set_cameraID, 0, NULL},
     {"/File/Take A Picture", "<control>T", menu_take_picture, 0, NULL},
     {"/File/Save Preference", "<control>S", menu_save_rc, 0, NULL},
     {"/File/---", NULL, NULL, 0, "<Separator>"},
     {"/File/Delete All", "<control>S", menu_del_all, 0, NULL},
     {"/File/---", NULL, NULL, 0, "<Separator>"},
     {"/File/Exit", "<control>X", menu_quit, 0, NULL},
     {"/Help", NULL, NULL, 0, "<Branch>"},
     {"/Help/About GKDCPI...", "<control>H", menu_about, 0, NULL}
};

static GtkItemFactoryEntry popup_items[] =
{
     {"/View Thumbnail", NULL, menu_view_thumb, 0, NULL},
     {"/Delete Image", NULL, menu_del_image, 0, NULL},
     {"/Save", NULL, menu_save_image, 0, NULL},
     {"/Save As...", NULL, menu_save_as, 0, NULL}
};

void init_menu()
{
  int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
  int npopup_items = sizeof(popup_items) / sizeof(popup_items[0]);
  GtkItemFactory *item_factory;
  GtkItemFactory *popup_item_factory;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<Main>", 
				       accel_group);
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  popup_item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<Popup>", NULL); 
  gtk_item_factory_create_items(popup_item_factory, npopup_items, popup_items, NULL);

  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

  menubar = gtk_item_factory_get_widget(item_factory, "<Main>");
  context_menu = gtk_item_factory_get_widget(popup_item_factory, "<Popup>");

  gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);
}
#endif

void menu_quit(GtkWidget *widget, gpointer data)
{
  do_exit(0);
}

void menu_sel_device(GtkWidget *widget, gpointer data)
{
   gint ret_val;

   ret_val = device_dialog();
   if (ret_val)
     {
	strcpy(device_str, "/dev/ttyS0");
	device_str[strlen(device_str) - 1] += device_no;
	gtk_label_set (GTK_LABEL (device), (gchar *)(device_str));
	gtk_widget_show (device);
     }
}

void menu_sel_ratio(GtkWidget *widget, gpointer data)
{
   gint ret_val;

   ret_val = ratio_dialog();
   if (ret_val)
     {
	switch( ratio_no )
	  {
	   case 9600:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"9600");
	     strcpy(ratio_str, "9600");
	     break ;
	   case 19200:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"19200");
	     strcpy(ratio_str, "19200");
	     break ;
	   case 38400:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"38400");
	     strcpy(ratio_str, "38400");
	     break ;
	   case 57600:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"57600");
	     strcpy(ratio_str, "57600");
	     break ;
	   case 115200:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"115200");
	     strcpy(ratio_str, "115200");
	     break ;
	   default:
	     gtk_label_set (GTK_LABEL (ratio), (gchar *)"9600");
	     strcpy(ratio_str, "9600");
	     break ;
	  }
	gtk_widget_show (ratio);
     }
}

void menu_save_rc(GtkWidget *widget, gpointer data)
{
  char *temp;
  FILE *f;

  if ((f = fopen (rcpath, "w")) != 0)
    {
      temp = safe_malloc(SHORT_STRING);
      sprintf(temp, "device:%s\n", device_str);
      fputs (temp, f);
      sprintf(temp, "ratio:%s\n", ratio_str);
      fputs (temp, f);
      sprintf(temp, "dir:%s\n", dir_str);
      fputs (temp, f);
      safe_free((void **)&temp);
      fclose(f);
    }

  alert_dialog("Preference is saved.", "OK", NULL, NULL);
}

void menu_about(GtkWidget *widget, gpointer data)
{
   char *temp;

   temp = safe_malloc(SHORT_STRING);
   sprintf(temp, "Kodak Digital Camera Utility on GTK+ ver %s\n\n", VERSION);
   strcat(temp, "Based on kdcpi by Brian A. Hirt\n");
   strcat(temp, "http://www.berkhirt.com/HomerProductions/products/kdcpi/\n\n");
   strcat(temp, "Copyright(C) Toru Hoshina\n");
   strcat(temp, "http://www.best.com/~hoshina\n");
   alert_dialog(temp, "Close", NULL, NULL);

   safe_free((void **)&temp);
}

void menu_view_thumb(GtkWidget *widget, gpointer data)
{
   int i, ret_val;
   struct pinfo_lst *plst;

   menus_enable_disable(FALSE);

   plst = proot;
   for ( i = 0 ; i < list_selected_row && plst != NULL ; i++)
     {
	plst = plst->next;
     }

   ret_val = get_thumbnail(i + 1, plst);

   if (ret_val == -1)
     {
	initialized = 0;
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
     } 

   menus_enable_disable(TRUE);
}

void menu_del_all(GtkWidget *widget, gpointer data)
{
   int ret_val;

   ret_val = alert_dialog("Are you sure??", "OK", "Cancel", NULL);
   if (ret_val == 0)
     return;

   menus_enable_disable(FALSE);

   ret_val = del_all_picture();
   if (ret_val == -1)
     {
	initialized = 0;
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
	return;
     } 
   acq_list(widget, data);

   menus_enable_disable(TRUE);
}

void menu_del_image(GtkWidget *widget, gpointer data)
{
   int ret_val;

   menus_enable_disable(FALSE);

   ret_val = del_picture(list_selected_row + 1);
   if (ret_val == -1)
     {
	initialized = 0;
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
     } 

   acq_list(widget, data);

   menus_enable_disable(TRUE);
}

void menu_save_image(GtkWidget *widget, gpointer data)
{
   int i, ret_val;
   struct pinfo_lst *plst;

   menus_enable_disable(FALSE);

   plst = proot;
   for ( i = 0 ; i < list_selected_row && plst != NULL ; i++)
     {
	plst = plst->next;
     }

   ret_val = get_picture(i + 1, plst, plst->filename);

   if (ret_val == -1)
     {
	initialized = 0;
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
     } 

   menus_enable_disable(TRUE);
}

void menu_save_as(GtkWidget *widget, gpointer data)
{
   menus_enable_disable(FALSE);
   file_selection(0);
}

void menu_get_status(GtkWidget *widget, gpointer data)
{
   int ret_val;

   if (initialized == 0)
     {
	ret_val = init_dc210();
	if (ret_val == -1)
	  return;
	ret_val = load_status();
	if (ret_val == -1)
	  return;
	initialized = 1;
     }

   status_dialog();
   menus_enable_disable(TRUE);
}

#ifndef GTK_HAVE_FEATURES_1_1_0
void menus_set_sensitive(char *path, int sensitive)
{
  GtkMenuPath *menu_path;

  if (initialize)
    menus_init ();

  menu_path = gtk_menu_factory_find (factory, path);
  if (menu_path)
    gtk_widget_set_sensitive (menu_path->widget, sensitive);
}
#else
void menus_set_sensitive(char *path, int sensitive)
{
  GtkItemFactory *factory;
  GtkWidget *menu_item;

  factory = gtk_item_factory_from_path (path);
  if (factory)
    {
       menu_item = gtk_item_factory_get_widget (factory, path);
       if (menu_item)
	 gtk_widget_set_sensitive (GTK_WIDGET(menu_item), sensitive);
    }
}
#endif

void menus_enable_disable(gint flag)
{
   if ((int)status.numPictures == 0)
     {
	gtk_widget_set_sensitive(GTK_WIDGET(savebutton), FALSE);
	menus_set_sensitive("<Main>/File/Delele All", FALSE);
     }
   else
     {
	gtk_widget_set_sensitive(GTK_WIDGET(savebutton), flag);
	menus_set_sensitive("<Main>/File/Delele All", flag);
     }
   gtk_widget_set_sensitive(GTK_WIDGET(listbutton), flag);
   menus_set_sensitive("<Main>/File/Get Status...", flag);
   menus_set_sensitive("<Main>/File/Set Camera ID...", flag);
   menus_set_sensitive("<Main>/File/Take A Picture", flag);
   menus_set_sensitive("<PopUp>/View Thumbnail", flag);
   menus_set_sensitive("<PopUp>/Delete Image", flag);
   menus_set_sensitive("<PopUp>/Save", flag);
   menus_set_sensitive("<PopUp>/Save As...", flag);
}

void menu_set_cameraID(GtkWidget *widget, gpointer data)
{
   char *temp;
   int ret_val;

   temp = safe_malloc(SHORT_STRING);
   ret_val = entry_dialog("Please input new camera identification", "OK", "Cancel", temp);
   if (ret_val != -1)
     {
	safe_free((void **)&temp);
	return;
     }

   ret_val = set_camera_id(temp);
   safe_free((void **)&temp);

   if (ret_val == -1)
     {
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
	return;
     }

   menus_enable_disable(TRUE);
}

static void
take_picture_cancel (void)
{
  progress_canceled = 1;
}

void menu_take_picture(GtkWidget *widget, gpointer data)
{
   int i, ret_val;
   int delay;

   progress_canceled = 0;

   menus_enable_disable(FALSE);

   ret_val = timer_dialog(&delay);
   if (ret_val != -1)
     {
	menus_enable_disable(TRUE);
	return;
     }

   if (delay != 0)
     {
	if (progress_box)
	  progress_free (progress_box);
	progress_box = progress_new ("Take A Picture", "Remaining time...",
				     (GtkSignalFunc) take_picture_cancel, 0);

	for ( i = delay ; i >= 0 ; i--)
	  {
	     while (gtk_events_pending ())
	       gtk_main_iteration ();
	     progress_update (progress_box, 
			      i / (gfloat)delay);
	     sleep(1);
	     while (gtk_events_pending ())
	       gtk_main_iteration ();

	     if (progress_canceled == 1)
	       {
		  menus_enable_disable(TRUE);
		  progress_free (progress_box);
		  progress_box = NULL;
		  return;
	       }
	  }
	progress_free (progress_box);
	progress_box = NULL;
     }

   ret_val = take_picture();
   if (ret_val == -1)
     {
	gtk_widget_set_sensitive(GTK_WIDGET(listbutton), TRUE);
	menus_set_sensitive("<Main>/File/Get Status...", TRUE);
	return;
     }
   acq_list(widget, data);

   menus_enable_disable(TRUE);
}
