/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* XPM plugin version 1.2.2 */

/* 1.2.2 fixes bug that generated bad digits on images with more than 20000 colors. (thanks, yanele)
parses gtkrc (thanks, yosh)
doesn't load parameter screen on images that don't have alpha

1.2.1 fixes some minor bugs -- spaces in #XXXXXX strings, small typos in code.

1.2 compute color indexes so that we don't have to use XpmSaveXImage*

Previous...Inherited code from Ray Lehtiniemi, who inherited it from S & P.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <math.h>

#include <gtk/gtk.h>

#include "xpm.h"

static const char linenoise [] =
" .+@#$%&*=-;>,')!~{]^/(_:<[}|1234567890abcdefghijklmnopqrstuvwxyz\
ABCDEFGHIJKLMNOPQRSTUVWXYZ`";

#define SCALE_WIDTH 125

typedef struct
{
   guchar r;
   guchar g;
   guchar b;
} rgbkey;

/* whether the image is color or not.  global so I only have to pass one user value
to the GHFunc */
int       color;
/* bytes per pixel.  global so I only have to pass one user value
to the GHFunc */
int        cpp;

guint rgbhash (rgbkey *c)
{
   return ((guint)c->r) ^ ((guint)c->g) ^ ((guint)c->b);
}

guint compare (rgbkey *c1, rgbkey *c2)
{
   return (c1->r == c2->r)&&(c1->g == c2->g)&&(c1->b == c2->b);
}
	
void set_XpmImage(XpmColor *array, guint index, char *colorstring)
{
   char *p;
   int i, charnum, indtemp;
	
   indtemp=index;
   array[index].string = p = g_new(char, cpp+1);
	
   /*convert the index number to base sizeof(linenoise)-1 */
   for(i=0; i<cpp; ++i)
     {
	charnum = indtemp%(sizeof(linenoise)-1);
	indtemp = indtemp / (sizeof (linenoise)-1);
	*p++=linenoise[charnum];
     }
   /* *p++=linenoise[indtemp]; */
	
   *p='\0'; /* C and its stupid null-terminated strings...*/
		
			
   array[index].symbolic = NULL;
   array[index].m_color = NULL;
   array[index].g4_color = NULL;
   if (color)
     {
	array[index].g_color = NULL;
	array[index].c_color = colorstring;
     } else {	
	array[index].c_color = NULL;
	array[index].g_color = colorstring;
     }
}

void create_colormap_from_hash (gpointer gkey, gpointer value, gpointer
				user_data)
{
   rgbkey *key = gkey;
   char *string = g_new(char, 8);
   sprintf(string, "#%02X%02X%02X", (int)key->r, (int)key->g, (int)key->b);
   set_XpmImage(user_data, *((int *) value), string);
}


gint save_xpm(char *filename, guchar *data)
{
   int       width;
   int       height;
   int 	    ncolors=1;
   int	    *indexno;
  
   XpmColor  *colormap;
   XpmImage  *image;
    
   guint   *ibuff   = NULL;
   /*guint   *mbuff   = NULL;*/

   guchar    *buffer;

   GHashTable *hash = NULL;
  
   int        j, k;
   //int        threshold = 255 * xpmvals.threshold;

   gint      rc = FALSE;

   color = 1;
   width    = 96;
   height   = 72;

   /* allocate buffers making the assumption that ibuff and mbuff
    are 32 bit aligned... */
   if ((ibuff = g_new(guint, width*height)) == NULL)
     goto cleanup;

   /*if ((mbuff = g_new(guint, width*height)) == NULL)
     goto cleanup;*/

   if ((hash = g_hash_table_new((GHashFunc)rgbhash, (GCompareFunc) compare)) == NULL)
     goto cleanup;

   /* allocate a pixel region to work with */
   if ((buffer = g_new(guchar, height*width*2)) == NULL)
     return 0;

   /* process each pixel row */
   for (j=0; j<height; j++)
     {
	/* go to the start of this row in each image */
	guint *idata = ibuff + j * width;
	/*guint *mdata = mbuff + (i+j) * width;*/

	/* do each pixel in the row */
	for (k=0; k<width; k++)
	  {
	     rgbkey *key = g_new(rgbkey, 1);
  
	     /* get pixel data */
	     key->r = *(data++);
	     key->g = *(data++);
	     key->b = *(data++);

	     indexno = g_hash_table_lookup(hash, key);
	     if (!indexno) {
		indexno = g_new(int, 1);
		*indexno = ncolors++;
		g_hash_table_insert(hash, key, indexno);
		key = g_new(rgbkey, 1);
	     }
	     *(idata++) = *indexno;
	  }
     }

   g_free(buffer);

   colormap = g_new(XpmColor, ncolors);
	  
   cpp=(double)1.0+(double)log(ncolors)/(double)log(sizeof(linenoise)-1.0);
   set_XpmImage(colormap, 0, "None");

   g_hash_table_foreach (hash, create_colormap_from_hash, colormap);

   image = g_new(XpmImage, 1);

   image->width=width;
   image->height=height;
   image->ncolors=ncolors;
   image->cpp=cpp;
   image->colorTable=colormap;	    
   image->data = ibuff;
  
   /* do the save */
   XpmWriteFileFromXpmImage(filename, image, NULL);


cleanup:

   if (ibuff) g_free(ibuff);
   /*if (mbuff) g_free(mbuff);*/
   if (hash)  g_hash_table_destroy(hash);

   return rc;
}
